(* ----------------------------------------------------------------------------
unit NLDVBulletinDelphiHighlighter
Download the latest version at
http://www.nldelphi.com/Forum/forumdisplay.php?s=&forumid=72
-------------------------------------------------------------------------------
Author:  Jos Visser aka GolezTrol
Date  :  february 2004
Web   :  www.goleztrol.nl
-------------------------------------------------------------------------------
Uses NLDDelphiHighlighter to output Delphi source to vBulletin mark-up
-------------------------------------------------------------------------------
Changes:    Version and description
----------  -------------------------------------------------------------------
2004-02-10  1.0: Created
2008-02-09  1.1: Quick fix for escaping vBulleting tags.
-------------------------------------------------------------------------------
ToDo:
- Optimizing the escape feature of version 1.1.
---------------------------------------------------------------------------- *)
unit NLDVBulletinDelphiHighlighter;

interface

uses
  Graphics, NLDDelphiHighlighter;

type
  TNLDvBDelphiHighlighter = class(TNLDCustomDelphiHighlighter)
  private
    FColor: TColor;
  protected
    procedure BeginDocument; override;
    procedure EndDocument; override;

    procedure BeginForegroundColor(Color: TColor); override;
    procedure EndForegroundColor(Color: TColor); override;
    procedure BeginBold; override;
    procedure EndBold; override;
    procedure BeginItalic; override;
    procedure EndItalic; override;

    procedure WriteText(const Text: string); override;
  public
    constructor Create; override;
    property TextStyles;
  end;

implementation

uses
  SysUtils;

{ TNLDvBDelphiHighlighter }

procedure TNLDvBDelphiHighlighter.BeginBold;
begin
  Write('[B]');
end;

procedure TNLDvBDelphiHighlighter.BeginDocument;
begin
  inherited;
  Write('[CODE]');
end;

procedure TNLDvBDelphiHighlighter.BeginForegroundColor(Color: TColor);
var
  Clr: Integer;
begin
  Clr := ColorToRGB(Color);
  FColor := Clr;
  Clr := Clr and $FF shl 16 + Clr and $FF00 + Clr and $FF0000 shr 16;
  Write(Format('[COLOR=#%6.6x]', [Clr] ));
end;

procedure TNLDvBDelphiHighlighter.BeginItalic;
begin
  Write('[I]');
end;

constructor TNLDvBDelphiHighlighter.Create;
begin
  inherited;
  FColor := clDefault;
end;

procedure TNLDvBDelphiHighlighter.EndBold;
begin
  Write('[/B]');
end;

procedure TNLDvBDelphiHighlighter.EndDocument;
begin
  Write('[/CODE]');
  inherited;
end;

procedure TNLDvBDelphiHighlighter.EndForegroundColor(Color: TColor);
begin
  FColor := clDefault;
  Write('[/COLOR]');
end;

procedure TNLDvBDelphiHighlighter.EndItalic;
begin
  Write('[/I]');
end;

procedure TNLDvBDelphiHighlighter.WriteText(const Text: string);
const
  Keywords: array[0..3] of string = ('i', 'b', 'color', 'code');
var
  s: string;
  c: string;
  Clr: Integer;
  i: Integer;
begin
  // When the text contains any [i] or any other of the above vB keywords
  // enclosed in brackets, include them in redundant color tags. This will
  // escape them so vBulleting won't recognise them als markup tags.
  s := Text;
  for i := Low(Keywords) to High(Keywords) do
  begin
    Clr := ColorToRGB(FColor);
    Clr := Clr and $FF shl 16 + Clr and $FF00 + Clr and $FF0000 shr 16;

    c := Format('[[COLOR=#%6.6x]%s[/COLOR]]', [Clr, Keywords[i]]);

    s := StringReplace(s, '['+Keywords[i]+']', c, [rfIgnoreCase, rfReplaceAll]);

  end;

  inherited WriteText(s);
end;

end.
